/* cmd_neonserv_peek.c - NeonServ v5.6
 * Copyright (C) 2011-2012  Philipp Kreil (pk910)
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License 
 * along with this program. If not, see <http://www.gnu.org/licenses/>. 
 */

#include "cmd_neonserv.h"

/*
* no parameters
*/
static USERLIST_CALLBACK(neonserv_cmd_peek_userlist_lookup);
static void neonserv_cmd_peek_async1(struct ClientSocket *client, struct ClientSocket *textclient, struct UserNode *user, struct ChanNode *chan);

struct neonserv_cmd_peek_cache {
    struct ClientSocket *client, *textclient;
    struct UserNode *user;
};

CMD_BIND(neonserv_cmd_peek) {
    struct neonserv_cmd_peek_cache *cache = malloc(sizeof(*cache));
    if (!cache) {
        perror("malloc() failed");
        return;
    }
    cache->client = client;
    cache->textclient = textclient;
    cache->user = user;
    get_userlist_if_invisible(chan, module_id, neonserv_cmd_peek_userlist_lookup, cache);
}

static USERLIST_CALLBACK(neonserv_cmd_peek_userlist_lookup) {
    struct neonserv_cmd_peek_cache *cache = data;
    neonserv_cmd_peek_async1(cache->client, cache->textclient, cache->user, chan);
    free(cache);
}

static void neonserv_cmd_peek_async1(struct ClientSocket *client, struct ClientSocket *textclient, struct UserNode *user, struct ChanNode *chan) {
    reply(textclient, user, "NS_PEEK_HEADER", chan->name);
    reply(textclient, user, "NS_PEEK_TOPIC", chan->topic);
    char tmpStr[MAXLEN];
    getModeString(chan->modes, tmpStr);
    reply(textclient, user, "NS_PEEK_MODES", tmpStr);
    struct ChanUser *chanuser;
    int with_halfops = get_int_field("General.have_halfop");
    int op_count = 0, halfop_count = 0, voice_count = 0, normal_count = 0, invi_count = 0;
    for(chanuser = getChannelUsers(chan, NULL); chanuser; chanuser = getChannelUsers(chan, chanuser)) {
        if(chanuser->flags & CHANUSERFLAG_OPPED)
            op_count++;
        else if(with_halfops && (chanuser->flags & CHANUSERFLAG_HALFOPPED))
            halfop_count++;
        else if(chanuser->flags & CHANUSERFLAG_VOICED)
            voice_count++;
        else if(chanuser->flags & CHANUSERFLAG_INVISIBLE)
            invi_count++;
        else
            normal_count++;
    }
    if(with_halfops)
        reply(textclient, user, "NS_PEEK_USERS_HALFOP", op_count+halfop_count+voice_count+invi_count+normal_count, op_count, halfop_count, voice_count, normal_count, invi_count);
    else
        reply(textclient, user, "NS_PEEK_USERS", op_count+voice_count+invi_count+normal_count, op_count, voice_count, normal_count, invi_count);
    int tmpStrPos = 0;
    int headerlen = 10 + strlen(user->nick);
    for(chanuser = getChannelUsers(chan, NULL); chanuser; chanuser = getChannelUsers(chan, chanuser)) {
        if(chanuser->flags & CHANUSERFLAG_OPPED) {
            if(tmpStrPos + headerlen + strlen(chanuser->user->nick) + 2 >= 512) {
                //clear buffer
                reply(textclient, user, "%s", tmpStr);
                tmpStrPos = 0;
            }
            tmpStrPos += sprintf(tmpStr + tmpStrPos, (tmpStrPos ? ", %s" : "%s"), chanuser->user->nick);
        }
    }
    if(tmpStrPos) {
        reply(textclient, user, "%s", tmpStr);
    }
}
